"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PulumiAdapter = void 0;
const path_1 = require("path");
const base_1 = require("@plutolang/base");
const automation_1 = require("@pulumi/pulumi/automation");
const utils_1 = require("./utils");
const fs_1 = require("fs");
const fs_extra_1 = require("fs-extra");
class PulumiAdapter extends base_1.core.Adapter {
    backendPath;
    passphrase;
    //eslint-disable-next-line @typescript-eslint/no-var-requires
    name = require((0, path_1.join)(__dirname, "../package.json")).name;
    //eslint-disable-next-line @typescript-eslint/no-var-requires
    version = require((0, path_1.join)(__dirname, "../package.json")).version;
    constructor(args) {
        if (!(0, path_1.isAbsolute)(args.entrypoint)) {
            throw new Error("The entrypoint provided to the adapter must be an absolute path.");
        }
        super(args);
        this.backendPath = (0, path_1.join)(base_1.utils.systemConfigDir(), "pulumi");
        (0, fs_extra_1.ensureDirSync)(this.backendPath);
        // this.passphrase = randomUUID();
        this.passphrase = "pluto";
    }
    async state() {
        try {
            const pulumiStack = await this.createPulumiStack();
            const stackState = await pulumiStack.workspace.stack();
            if (!stackState) {
                throw new Error("Cannot find the target pulumi stack. Have you already deployed it?");
            }
            await pulumiStack.refresh();
            const result = await pulumiStack.exportStack();
            const instances = [];
            const resources = result.deployment.resources ?? [];
            for (const resource of resources) {
                const instance = {
                    id: resource["urn"],
                    name: resource["urn"].split("::").pop(),
                    type: resource["type"],
                    status: base_1.core.ResourceInstanceStatus.Deployed,
                    parent: resource["parent"],
                };
                instances.push(instance);
            }
            return { instances };
        }
        catch (e) {
            if (e instanceof Error) {
                throw new Error("Met error during run 'pulumi state', " + e.message);
            }
            else {
                throw new Error("Met error during run 'pulumi state', " + e);
            }
        }
    }
    async deploy(opts = {}) {
        try {
            const pulumiStack = await this.createPulumiStack();
            if (this.updateInProgress()) {
                if (opts.force) {
                    await pulumiStack.cancel();
                }
                else {
                    throw new Error("This stack is currently being updated. If you want to update forcefully, you can use the --force option.");
                }
            }
            const result = await pulumiStack.up();
            return { outputs: result.outputs["default"].value };
        }
        catch (e) {
            if (e instanceof Error) {
                throw new Error("Met error during run 'pulumi update', " + e.message);
            }
            else {
                throw new Error("Met error during run 'pulumi update', " + e);
            }
        }
    }
    async destroy(opts = {}) {
        try {
            const pulumiStack = await this.createPulumiStack();
            if (this.updateInProgress()) {
                if (opts.force) {
                    await pulumiStack.cancel();
                }
                else {
                    throw new Error("This stack is currently being updated. If you want to update forcefully, you can use the --force option.");
                }
            }
            await pulumiStack.refresh();
            await pulumiStack.destroy();
            await pulumiStack.workspace.removeStack(this.stack.name);
        }
        catch (e) {
            if (e instanceof Error) {
                throw new Error("Met error during run 'pulumi destroy', " + e.message);
            }
            else {
                throw new Error("Met error during run 'pulumi destroy', " + e);
            }
        }
    }
    dump() {
        return JSON.stringify({
            passphrase: this.passphrase,
        });
    }
    load(data) {
        const config = JSON.parse(data);
        this.passphrase = config["passphrase"];
    }
    async createPulumiStack() {
        const envs = {
            RUNTIME_TYPE: this.stack.platformType,
            ENGINE_TYPE: this.stack.engineType,
            PULUMI_CONFIG_PASSPHRASE: this.passphrase,
            WORK_DIR: this.workdir,
        };
        const pulumiStack = await automation_1.LocalWorkspace.createOrSelectStack({
            stackName: this.stack.name,
            workDir: this.workdir,
        }, {
            workDir: this.workdir,
            envVars: envs,
            projectSettings: {
                runtime: "nodejs",
                name: this.project,
                main: this.entrypoint,
                backend: { url: "file://" + this.backendPath },
            },
        });
        for (const key of Object.keys(envs))
            process.env[key] = envs[key];
        const pulumiConfig = await (0, utils_1.genPulumiConfigByRuntime)(this.stack);
        await pulumiStack.setAllConfig(pulumiConfig);
        return pulumiStack;
    }
    updateInProgress() {
        const locksDir = (0, path_1.resolve)(this.backendPath, ".pulumi/locks/organization", this.project, this.stack.name);
        const files = (0, fs_1.readdirSync)(locksDir);
        return files.length > 0;
    }
}
exports.PulumiAdapter = PulumiAdapter;
//# sourceMappingURL=pulumi.js.map